--------------------------------------------------------------------------------
-- Copyright (c) 2002 by Trenz Electronic GmbH.
-- Brendel 20, 32257 Buende, Germany, www.trenz-electronic.de
--------------------------------------------------------------------------------
-- Project:     TE-XC2Se Spartan-IIe Development System.
-- File:        led.vhd
-- Description: Modulates LED5 on TE-XC2Se.
-- History:     2002may8, TT, created.
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library work;
use work.free_dac_lib.dac_pwm;


entity led is
    Port ( 
		clk48:    in  std_logic;							-- master clock
		rst:      in  std_logic;							-- master reset
		led5:     out std_logic								-- LED output
	);
end led;
--------------------------------------------------------------------------------

architecture Behavioral of led is
	constant size: integer := 16;							-- size of pwm counter
	
	signal cnt:		STD_LOGIC_VECTOR(size-1 downto 0);		-- intensity counter
	signal clk:		std_logic;								-- low frequency intensity clock
	signal div:		unsigned(10 downto 0);					-- counter for low frequency clock 
	signal dir:		std_logic;								-- intensity up or down

begin
	process(clk48, rst)
	-- generate low frequency clock
	begin
		if rst= '1' then
			div<= (others=> '0');
		elsif rising_edge(clk48) then
			div<= div + 1;
		end if;
	end process;

	clk<= div(div'left);


	pwm:dac_pwm						
		generic map (size)
		port map (rst, clk48, cnt, led5);


	process (clk, rst)
	-- up/down counter for intensity
	begin
		if rst='1' then
			cnt <= x"0000";
			dir <= '1';
		elsif rising_edge(clk) then
			if cnt = x"0002" then dir <= '1'; end if;        
			if cnt = x"D000" then dir <= '0'; end if;       
			if dir = '1' then cnt <= cnt + 1; end if;
			if dir = '0' then cnt <= cnt - 1; end if;
		end if;	
	end process;

end Behavioral;
